@@ -47,16 +47,23 @@ def append_fcns(fcn,*fcns):
             return False
     return rtn
 
-def create_logger(name,level='INFO',add_stdout_handler=True):
+def create_logger(name,level='INFO',format='[%(asctime)s] [%(name)s] [%(levelname)s] %(message)s',
+                change_if_exists=False,add_stdout_handler=False):
+    logger_exists = name in logging.getLogger().manager.loggerDict
     logger = logging.getLogger(name)
-    level=getattr(logging,level)
-    logger.setLevel(level)
 
-    ch = logging.StreamHandler() #logging.StreamHandler(sys.stdout)
-    ch.setLevel(level)
-    formatter = logging.Formatter('[%(asctime)s] [%(name)s] [%(levelname)s] %(message)s')
-    ch.setFormatter(formatter)
-    logger.addHandler(ch)
+    if not logger_exists or change_if_exists:
+        logging.basicConfig(format=format)
+        level=getattr(logging,level)
+        logger.setLevel(level)
+
+        if add_stdout_handler:
+            ch = logging.StreamHandler(sys.stdout)
+            if level is not None:
+                ch.setLevel(level)
+            formatter = logging.Formatter(format)
+            ch.setFormatter(formatter)
+            logger.addHandler(ch)
     return logger
 
 def create_uuid(hex=True):
